Search.setIndex({docnames:["api","api/cards","api/errors","index","intro"],envversion:53,filenames:["api.rst","api/cards.rst","api/errors.rst","index.rst","intro.rst"],objects:{},objnames:{},objtypes:{},terms:{api:3,content:3,index:3,modul:3,packag:3,page:3,provid:3,refer:3,search:3,thi:3},titles:["API Reference","&lt;no title&gt;","&lt;no title&gt;","AnkiVim","&lt;no title&gt;"],titleterms:{ankivim:3,api:0,document:[],indic:3,refer:0,tabl:3,welcom:[]}})